-- File................: Top_Level.vhd
-- Function............: High Speed Sample Functionality
-- Person Responsible..: Actel Corporation
-- Version.............: 1.00
-- Last Updated........: 07-14-2004
-- Component of........: 
-- Components Required.: 
-- Compilation Notes...:
--
-- Revision History:
-- 

-- Functional Description:
--
 
LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
LIBRARY axcelerator;                   --Required for simulation due to instanciated clkints
USE Axcelerator.components.all;        

entity Top_Level is

  port(
       DATA_IN    : in std_logic;
       RST        : in std_logic;
 --      CLK100Mhz  : in std_logic;
       CLK1       : in std_logic;
       CLK2       : in std_logic;
       CLK3       : in std_logic;
       CLK4       : in std_logic;
       PLLa_PWRDN : in std_logic;
       PLLb_PWRDN : in std_logic;
       PLLc_PWRDN : in std_logic;
       PLLd_PWRDN : in std_logic;
       Mem_data   : out std_logic_vector(7 downto 0);
       Rd_enable  : out std_logic;
       LOCK_PLLa  : out std_logic;
       LOCK_PLLb  : out std_logic;
       LOCK_PLLc  : out std_logic;
       LOCK_PLLd  : out std_logic
       );

end Top_Level;

architecture rtl of Top_Level is

-----------------------------------------------------------
-- SIGNAL DECLARATIONS
-----------------------------------------------------------
  signal BS1_CLK : std_logic;
  signal BS2_CLK : std_logic;
  signal BS3_CLK : std_logic;
  signal BS4_CLK : std_logic;
  signal ClkA    : std_logic;
  signal ClkB    : std_logic;
  signal ClkC    : std_logic;
  signal ClkD    : std_logic;
  signal DataA   : std_logic;
  signal DataB   : std_logic;
  signal DataC   : std_logic;
  signal DataD   : std_logic;
  signal DataE   : std_logic;
  signal DataF   : std_logic;
  signal DataG   : std_logic;
  signal DataH   : std_logic;
  signal DataABCD  : std_logic_vector(3 downto 0);
  signal DataEFGH  : std_logic_vector(3 downto 0);
  signal DataABCDEFGH: std_logic_vector(7 downto 0);
 -- signal Mem_data: std_logic_vector(7 downto 0);


-----------------------------------------------------------
-- COMPONENT DECLARATIONS
-----------------------------------------------------------

component pll_a
    port(
      REFCLK   : in std_logic;  
      PWRDN    : in std_logic;  
      CLK500   : out std_logic; 
      LOCK     : out std_logic
      ) ;
end component;

component pll_b
    port(
       REFCLK : in std_logic; 
       PWRDN  : in std_logic;  
       CLK500 : out std_logic; 
       LOCK   : out std_logic) ;
end component;

component pll_c
    port(
       REFCLK : in std_logic; 
       PWRDN  : in std_logic;  
       CLK500 : out std_logic; 
       LOCK   : out std_logic) ;
end component;

component pll_d
    port(
       REFCLK : in std_logic; 
       PWRDN  : in std_logic;  
       CLK500 : out std_logic; 
       LOCK   : out std_logic) ;
end component;


component inbuf
   port(
      PAD                            :	in    STD_ULOGIC;
      Y                              :	out   STD_ULOGIC);
end component; 


begin

--Latch the data into register A on the rising edge of CLKA

REGA :  process (CLKA, RST)
    begin
      if (RST = '0') then
         DataA <= '0';
      elsif (CLKA'event and CLKA = '1') then
         DataA <= DATA_IN;
      end if;
  end process;

--Latch the data into register B on the rising edge of CLKB

REGB :  process (CLKB, RST)
    begin
      if (RST = '0') then
         DataB <= '0';
      elsif (CLKB'event and CLKB = '1') then
         DataB <= DATA_IN;
      end if;
  end process;

--Latch the data into register C on the rising edge of CLKC

REGC :  process (CLKC, RST)
    begin
      if (RST = '0') then
         DataC <= '0';
      elsif (CLKC'event and CLKC = '1') then
         DataC <= DATA_IN;
      end if;
  end process;


--Latch the data into register D on the rising edge of CLKD

REGD :  process (CLKD, RST)
    begin
      if (RST = '0') then
         DataD <= '0';
      elsif (CLKD'event and CLKD = '1') then
         DataD <= DATA_IN;
      end if;
  end process;


--Latch the data into a register on the falling edge of CLKA 

REGE :  process (CLKA, RST)
    begin
      if (RST = '0') then
         DataE <= '0';
      elsif (CLKA'event and CLKA = '0') then
         DataE <= DATA_IN;
      end if;
  end process;

--Latch the data into a register on the falling edge of CLKB

REGF :  process (CLKB, RST)
    begin
      if (RST = '0') then
         DataF <= '0';
      elsif (CLKB'event and CLKB = '0') then
         DataF <= DATA_IN;
      end if;
  end process; 

--Latch the data into a register on the falling edge of CLKC 

REGG :  process (CLKC, RST)
    begin
      if (RST = '0') then
         DataG <= '0';
      elsif (CLKC'event and CLKC = '0') then
         DataG <= DATA_IN;
      end if;
  end process;

--Latch the data into a register on the falling edge of CLKD 

REGH :  process (CLKD, RST)
    begin
      if (RST = '0') then
         DataH <= '0';
      elsif (CLKD'event and CLKD = '0') then
         DataH <= DATA_IN;
      end if;
  end process;

REGABCD : process (CLKA, RST)
    begin
      if (RST = '0') then
         DataABCD <= "0000";
      elsif (CLKA'event and CLKA = '1') then
         DataABCD <= DataA & DataB & DataC & DataD;
      end if;
  end process; 

REGEFGH : process (CLKA, RST)
    begin
      if (RST = '0') then
         DataEFGH <= "0000";
      elsif (CLKA'event and CLKA = '0') then
         DataEFGH <= DataE & DataF & DataG & DataH;
      end if;
  end process; 


REGABCDEFGH : process (CLKA, RST)
    begin
      if (RST = '0') then
         DataABCDEFGH <= "00000000";
      elsif (CLKA'event and CLKA = '1') then
         DataABCDEFGH <= DataABCD & DataEFGH;
      end if;
  end process; 

  Mem_data <= DataABCDEFGH;


CK1 :  inbuf    port map(
          PAD  => CLK1,
          Y    => BS1_CLK
          ); 
      
CK2 :  inbuf   port map( 
          PAD  => CLK2,
          Y    => BS2_CLK
          ); 

CK3 :  inbuf   port map( 
          PAD  => CLK3,
          Y    => BS3_CLK
          ); 

CK4 :  inbuf   port map( 
          PAD  => CLK4,
          Y    => BS4_CLK
          ); 

PLL1 :  pll_a   port map(
           REFCLK   => BS1_CLK,  
           PWRDN    => PLLa_PWRDN, 
           CLK500   => CLKA,  
           LOCK     => LOCK_PLLa
           );

PLL2 :  pll_b  port map(
           REFCLK => BS2_CLK,  
           PWRDN  => PLLb_PWRDN, 
           CLK500 => CLKB, 
           LOCK   => LOCK_PLLb
           );

PLL3 :  pll_c  port map(
           REFCLK => BS3_CLK,  
           PWRDN  => PLLc_PWRDN, 
           CLK500 => CLKC, 
           LOCK   => LOCK_PLLc
           );
  
PLL4 :  pll_d  port map(
           REFCLK => BS4_CLK,  
           PWRDN  => PLLd_PWRDN, 
           CLK500 => CLKD, 
           LOCK   => LOCK_PLLd
           );

end rtl;
